/********************************************************************************\
*  This sample is supplied as is with no implied warranty.  
*  It is designed to assist you in using the Cisco AnyConnect VPN API. 
*  It is assumed that you will build a production application and 
*  refer to this sample as a reference only.
\********************************************************************************/
#pragma once

#include <map>
#include <list>
#include "afxwin.h"
#include "afxcmn.h"
#include "EventCallBack.h"
#include "CppComSampleOtherDlgs.h"


// CCppComSampleDlg dialog
class CCppComSampleDlg : public CDialog, 
                         protected CComObjectStackEx<CEventCallBack> //This is the way you bind 
                                                                     //the COM Callbacks to the
                                                                     //Dialog class.
{
public:
    CCppComSampleDlg(CWnd* pParent = NULL);
	enum { IDD = IDD_COMSAMPLE_DIALOG };


protected:
    enum
    {
        WM_VPN_EVENT_AVAILABLE       = WM_USER + 1,
        WM_VPN_REGISTRY_NOTIFICATION,
        WM_VPN_GROUP_CHANGED, 
        WM_VPN_BANNER_AVAILABLE,
        WM_VPN_PRECONNECT_REMINDER_AVAILABLE,
        WM_VPN_CERT_BLOCKED,
        WM_VPN_CERT_WARNING
    };

    virtual void DoDataExchange(CDataExchange* pDX);
	virtual BOOL OnInitDialog();

    //Methods added for initializing/shutting down the COM API.
    //
    HRESULT StartupCOMVpnApi(void);
    HRESULT ShutdownCOMVpnApi(void);

protected:
	// Generated message map functions
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
    afx_msg void OnDestroy();
	DECLARE_MESSAGE_MAP()

public:
    afx_msg void OnBnClickedButtonShowVpnStats();
    afx_msg void OnBnClickedShowRouteDetail();
    afx_msg void OnBnClickedOptions();
    afx_msg void OnHostsComboBoxSelectionChanged();
    afx_msg void OnBnClickedButtonSubmit();
    afx_msg void OnBnClickedButtonShowProtocol();
    afx_msg void OnPromptComboBoxSelectionChanged();
    afx_msg void OnBnClickedFirewallrules();
    afx_msg void OnBnClickedButtonConnect();

///////////// *BEGIN* COM EVENT METHODS THAT YOU IMPLEMENT ////////////////
protected:

    //IVpnApiEvents COM callback methods
    //It is assumed that COM was initialized as an STA.
    STDMETHOD(raw_VpnStatsNotification)(/*[in]*/ IVpnStats * pVpnStats);
    
    STDMETHOD(raw_VpnStateNotification)(/*[in]*/ enum VPNState eState,
                                        /*[in]*/ enum VPNSubState eSubState,
                                        /*[in]*/ BSTR rawbstrState );
    
    STDMETHOD(raw_VpnBannerNotification)(/*[in]*/ BSTR rawbstrBannerMessage);

    STDMETHOD(raw_VpnPreConnectReminderNotification)(/*[in]*/ BSTR rawbstrPreConnectReminderMessage);

    STDMETHOD(raw_VpnNoticeNotification)(/*[in]*/ IN BSTR rawbstrNoticeMessage, 
                                         /*[in]*/ enum MessageType eMessageType);

    STDMETHOD(raw_VpnExitNotification)(/*[in]*/ BSTR rawbstrExitMessage,
                                       /*[in]*/ long exitCode );
    
    STDMETHOD(raw_VpnServiceReadyNotification)(void);

    STDMETHOD(raw_VpnUserPromptNotification)(/*[in]*/ IConnectPromptInfo * pConnectPromptInfo);

    STDMETHOD(raw_VpnWMHintNotification)(/*[in]*/ enum WMHint eHint,
                                         /*[in]*/ enum WMHintReason eReason); 

    STDMETHOD(raw_VpnWebLaunchHostNotification)(/*[in]*/ BSTR rawbstrActiveHost);

    STDMETHOD(raw_VpnCertBlockedNotification)(/*[in]*/ BSTR rawbstrUntrustedServer);

    STDMETHOD(raw_VpnCertWarningNotification)(/* [in] */ BSTR rawbstrUntrustedServer,
                                              /* [in] */ IStringCollection *pCertErrors,
                                              /* [in] */ VARIANT_BOOL bImportAllowed);

    //This is only called if you explicity configured consumer driven event model.
    //See README_FIRST.htm for more details
    STDMETHOD(raw_VpnEventAvailableNotification)(void);

///////////// *END* COM EVENT METHODS THAT YOU IMPLEMENT ////////////////

    //This will be called when WM_VPN_EVENT_AVAILABLE is posted to the message queue
    //as a result of a raw_VpnEventAvailableNotification being fired.
    LRESULT OnVpnEventAvailable(WPARAM /*not used*/, LPARAM /*not used*/);


    //This will be called when WM_VPN_REGISTRY_NOTIFICATION is posted to the message queue
    //as a result of a registry notification getting signalled for the
    //the vpn software update status.
    //
    LRESULT OnVpnRegistryNotification(WPARAM bTimedOut, LPARAM /*not used*/);

    //This will be called whem WM_VPN_GROUP_CHANGED is posted to the message queue
    //signalling the GUI to recreate the credential controls
    //
    LRESULT OnGroupChanged(WPARAM /*not used*/, LPARAM /*not used*/);

    //Called as a result of a banner arrival
    //
    LRESULT OnBannerAvailableNotification(WPARAM /*not used*/, LPARAM /*not used*/);

    //Called as a result of a pre-connect reminder arrival
    //
    LRESULT OnPreConnectReminderAvailableNotification(WPARAM /*not used*/, LPARAM /*not used*/);

    //Called as a result of an untrusted server cert being blocked
    //
    LRESULT OnCertBlockedNotification(WPARAM /*not used*/, LPARAM /*not used*/);

    //Called as a result of an untrusted server cert when connections to untrusted VPN
    //servers are allowed
    //
    LRESULT OnCertWarningNotification(WPARAM /*not used*/, LPARAM /*not used*/);

private:
    void ResetStrings(void);
    HRESULT BuildHostComboBoxEntries(void);
    void ResetCredentialsUI(void);
    void DisableCredentialsUI(void);
    HRESULT StoreCredentialsUI(CComPtr<IConnectPromptInfo> & spConnectPromptInfo);
    HRESULT BuildCredentialsUI(CComPtr<IConnectPromptInfo> & spConnectPromptInfo);

    void CreateLabel(const CComQIPtr<IPromptEntry> & spPromptEntry, CRect & rectLocation);
    void CreateEditPrompt(const CComQIPtr<IPromptEntry> & spPromptEntry, CRect & rectLocation);
    void CreateComboPrompt(const CComQIPtr<IPromptEntry> & spPromptEntry, CRect & rectLocation);

    //called by the OS when the registry key being monitored has changed, or a timeout occurred.
    static void CALLBACK RegistryWaitCallback(void * pThis, BOOLEAN bTimedOut);

///Variables///
protected:
	HICON m_hIcon;
    HICON m_hOptionsIcon;

private:
    CComPtr<IVpnApi> m_spVpnApi; //smart-pointer to the COM API instance

    CComPtr<IConnectPromptInfo> m_spConnectPromptInfo; //connect prompt info object.  This is valid
                                                       //until m_spVpnApi->UserSubmit(); is called.

    VPNState m_currentVpnState;
    CString m_stateStr;
    CString m_serviceStr;
    CDlgStatusBar m_StatusBar;
    UINT m_IDArray[1]; 

    CVPNStatsDlg m_StatsDlg;
    CVPNRoutesDlg m_RoutesDlg;
    CVPNProtocolDlg m_ProtocolDlg;
    CVpnFirewall m_FirewallDlg;

    CComboBox m_HostSelectionComboBox;
    CButton   m_OptionsButton;

    CButton   m_SubmitButton;
    CButton m_ConnectButton;

    typedef std::map<_bstr_t, CWnd *> ControlMap;
    ControlMap m_promptLabels;
    ControlMap m_promptInputControls;

    //variables associated with vpn software upgrade-complete notifications.
    //these are used to monitor when the vpn software upgrade has completed.
    //
    CEvent    m_eventUpgradeComplete;
    HANDLE    m_hWatchHandle;
    CRegKey   m_keyUpgrade;
    CString   m_strUpgradeValueName;
    DWORD     m_dwPreviousUpgradeState;

    // cached banner message
    CString m_strBannerMsg;

    // cached pre-connect reminder message
    CString m_strPreConnectReminderMsg;

    CString m_strUntrustedServer;
    std::list<CString> m_lCertErrors;
    bool m_bImportAllowed;
};
